/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2005  Luca Bellonda <lbell at tsc4.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 #include <config.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <glib.h>
#include <gmodule.h>
#include <gtk/gtkmain.h>
#include <gtk/gtkiconfactory.h>
#include <gtk/gtkitemfactory.h>
#include <gtk/gtkstock.h>
#include <gtk/gtkhpaned.h>
#include <gtk/gtkframe.h>
#include <gtk/gtktreeview.h>
#include <gtk/gtkcellrenderertext.h>
#include <gtk/gtktreeselection.h>
#include <gtk/gtktreemodel.h>
#include <gtk/gtkscrolledwindow.h>
#include <gtk/gtkmessagedialog.h>
#include <libgnome/gnome-i18n.h>
#include <bonobo/bonobo-ui-component.h>
#include <bonobo/bonobo-ui-util.h>
#include <bonobo/bonobo-control.h>
#include <libplanner/mrp-task.h>
#include "planner-view.h"
#include "planner-cell-renderer-date.h"
#include "planner-task-dialog.h"
#include "planner-resource-dialog.h"
#include "planner-gantt-model.h"
#include "planner-task-tree.h"
#include "planner-gantt-chart.h"
#include "planner-gantt-print.h"
#include "planner-col-spc.h"

#define	GCONF_PATH_TOTAL	"/apps/planner/views/%s/task_columns/columns"
#define	GCONF_PATH_COLUMN_N	"/apps/planner/views/%s/task_columns/Column%d"

/* Macro for debugging */
#define	D(x)

#define	LABEL(x)	view_column_get_default_label(x)

#define	GCONF_UID_KEY	"ColumnId"
#define	GCONF_TITLE_KEY	"Title"

const gchar *view_column_get_default_label(const ColTaskEnum uid)
{
	switch(uid)
	{
	default: return _("?");
 	case COL_TM_WBS: return _("WBS");
	case COL_TM_NAME: return _("Name");
	case COL_TM_START: return _("Start");
	case COL_TM_FINISH: return _("Finish");
	case COL_TM_DURATION: return _("Duration");
	case COL_TM_WORK: return _("Work");
	case COL_TM_SLACK: return _("Slack");
	case COL_TM_WEIGHT: return _("Weight");
	case COL_TM_EDITABLE: return _("Editable");
	case COL_TM_TASK: return _("Task");
	case COL_TM_COST: return _("Cost");
	case COL_TM_RESOURCE: return _("Resource");
	case COL_TM_CONSTRAINT: return _("Contstraint");
	case COL_TM_NOTE: return _("Notes");
	case COL_TM_PERCENTDONE: return _("% done");
	case COL_TM_CARDINE: return _("Cardine");
	case COL_TM_FIXEDSTARTDATE: return _("Fixed Start Date");
	case COL_TM_STATEICON: return _("State");
	}
}

void view_column_def_init( ViewColumnDef *self, const ColTaskEnum uid, const gchar *title, const gboolean editable, const gfloat alignment, const int width )
{
	g_assert(NULL!=self);
	self->uid       = uid; 
	if(NULL==title)
		self->title     = g_strdup(view_column_get_default_label(uid));
	else
		self->title     = g_strdup(title);
	self->editable  = editable ;
	self->alignment = alignment ;
	self->width     = width;
}

ViewColumnDef *view_column_new( const ColTaskEnum uid, const gchar *title, const gboolean editable, const gfloat alignment, const int width )
{
	ViewColumnDef *new_def = (ViewColumnDef*)g_new0(ViewColumnDef,1);
	if(NULL!=new_def)
		view_column_def_init( new_def, uid, title, editable, alignment, width );
	return new_def;
}

ViewColumnDef *view_column_new_void()
{
	ViewColumnDef *new_def = (ViewColumnDef*)g_new0(ViewColumnDef,1);
	if(NULL!=new_def)
		view_column_def_init( new_def, COL_TM_INVALID, _(""), TRUE, .5, 100 );
	return new_def;
}

ViewColumnDef *view_column_clone( ViewColumnDef *self )
{
	ViewColumnDef *new_def = (ViewColumnDef*)g_new0(ViewColumnDef,1);
	if(NULL!=new_def)
		view_column_def_init( new_def, self->uid, self->title, self->editable, self->alignment, self->width );
	return new_def;
}

gboolean view_columns_save_conf( ViewColumnDef *self, GConfClient *gconf_client, const gchar * base_path )
{
	gboolean ok = TRUE;
	gchar *str_path ;
	g_assert(NULL!=self);
	str_path = g_strdup_printf( "%s/%s", base_path, GCONF_UID_KEY );
D(printf( "value is --%s--\n", str_path);)
	if(!gconf_client_set_int (gconf_client, str_path, self->uid, NULL ))
		ok = FALSE;
	g_free(str_path);
	str_path = g_strdup_printf( "%s/%s", base_path, GCONF_TITLE_KEY );
D(printf( " saving value is --%s--%s-- \n", str_path,self->title);)
	if(!gconf_client_set_string (gconf_client, str_path, self->title, NULL ))
		ok = FALSE ;
	g_free(str_path);
	return ok;
}

/* It deletes the column if the read value is invalid */
gboolean view_columns_read_conf( ViewColumnDef *self, GConfClient *gconf_client, const gchar * base_path )
{
	gchar *str_path ;
	g_assert(NULL!=self);
	str_path = g_strdup_printf( "%s/%s", base_path, GCONF_UID_KEY );
	self->uid = gconf_client_get_int (gconf_client, str_path, NULL );
	g_free(str_path);
	str_path = g_strdup_printf( "%s/%s", base_path, GCONF_TITLE_KEY );
	g_free(self->title);
	self->title = gconf_client_get_string (gconf_client, str_path, NULL );
	g_free(str_path);
	if((self->uid<COL_TM_START_BASE_COLUMN)||(self->uid>=COL_TM_NUM_COLS))
	{
		return FALSE ;
	}
	return TRUE;
}

void view_column_set_title( ViewColumnDef *self, const gchar *newTitle )
{
	//g_assert(NULL!=self);
	//g_warning(NULL!=newTitle);
	if(NULL!=self->title)
	{
		g_free(self->title);
		self->title = NULL ;
	}
	if(NULL!=newTitle)
		self->title = g_strdup(newTitle);
	else
		self->title = g_strdup("");
}

gchar* view_column_get_title_transl_dup( ViewColumnDef *self )
{
	if(NULL!=self->title)
	{
		if( self->title[0] == '\n' )
			return g_strdup_printf( "%s", &self->title[1] );
		return g_strdup(self->title);
	} else
		return g_strdup("");
}
/* returns the textual description of the column */
const gchar *view_column_get_type_string_const(ViewColumnDef *self)
{
	return view_column_get_default_label(self->uid);
}

void view_column_end( ViewColumnDef *self )
{
	g_assert(NULL!=self);
	if( NULL!=self)
	{
		g_free(self->title);
		self->title = NULL ; /* to debug */
		g_free(self);
	}
}


GList *view_column_get_all()
{
	GList    *col_list = NULL;/* relies on g_abort */
 	col_list = g_list_append (col_list, view_column_new(COL_TM_WBS,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_NAME,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_START,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_FINISH,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_DURATION, NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_WORK, NULL, FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_SLACK,NULL,FALSE,TV_ALIGN_LEFT,100));
	/*col_list = g_list_append (col_list, view_column_new(COL_TM_WEIGHT,NULL,FALSE,TV_ALIGN_LEFT,100));*/
	/*col_list = g_list_append (col_list, view_column_new(COL_TM_EDITABLE: return "COL_TM_EDITABLE";*/
	/*col_list = g_list_append (col_list, view_column_new(COL_TM_TASK,NULL,FALSE,TV_ALIGN_LEFT,100));*/
	col_list = g_list_append (col_list, view_column_new(COL_TM_COST,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_RESOURCE,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_CONSTRAINT, NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_NOTE,NULL,FALSE,TV_ALIGN_LEFT,100));
	/*col_list = g_list_append (col_list, view_column_new(COL_TM_PERCENTDONE: return "COL_TM_PERCENTDONE";
	col_list = g_list_append (col_list, view_column_new(COL_TM_CARDINE: return "COL_TM_CARDINE";
	col_list = g_list_append (col_list, view_column_new(COL_TM_FIXEDSTARTDATE: return "COL_TM_FIXEDSTARTDATE";
	col_list = g_list_append (col_list, view_column_new(COL_TM_STATEICON: return "COL_TM_STATEICON";*/

	return col_list ;
}

void view_columns_end(GList *col_list)
{
	GList *l;
	if(NULL==col_list);
		return ;
	for (l = col_list; NULL != l; l = l->next) {
		view_column_end( (ViewColumnDef*) l->data );
	}
	if(NULL!=col_list)
		g_list_free(col_list);
}

GList *view_columns_duplicate(GList *col_list)
{
	GList *l;
	GList    *cloned_list = NULL;
	g_assert(NULL!=col_list);
	for (l = col_list; NULL != l; l = l->next) {
		ViewColumnDef *to_clone = (ViewColumnDef*) l->data ;
		cloned_list = g_list_append (cloned_list , view_column_new(to_clone->uid, to_clone->title,
					to_clone->editable, to_clone->alignment, to_clone->width ) );
	}
	return cloned_list ;
}

GList *view_columns_get_default_task_init()
{
	GList    *col_list = NULL;
	col_list = g_list_append (col_list, view_column_new(COL_TM_WBS,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_NAME,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_START,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_FINISH,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_WORK, NULL, FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_SLACK,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_COST,NULL,FALSE,TV_ALIGN_LEFT,100));
	return col_list ;
}

/* Build the default tree for the gantt view */
GList *view_columns_get_default_gantt_init()
{
	GList    *col_list = NULL;/* relies on g_abort */
	col_list = g_list_append (col_list, view_column_new(COL_TM_NOTE,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_WBS,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_NAME,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_START,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_WORK,  NULL, FALSE,TV_ALIGN_LEFT,100));
/*	col_list = g_list_append (col_list, view_column_new(*COL_TM_DURATION, _("Duration"),*****s*/
	col_list = g_list_append (col_list, view_column_new(COL_TM_CONSTRAINT,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_SLACK,NULL,FALSE,TV_ALIGN_LEFT,100));
	col_list = g_list_append (col_list, view_column_new(COL_TM_RESOURCE,NULL,FALSE,TV_ALIGN_LEFT,100));
	return col_list ;
}

/* Reads the configuration from a file. Call it with: planner_view_get_name (view) */
gboolean
task_columns_save_configuration( GConfClient *gconf_client, const gchar *view_name, GList *columns_list )
{
	gboolean ok = TRUE;
	GList   *l;
	int	 column_index = 0;
	gchar   *str=NULL;

	g_assert(NULL!=columns_list);
	g_assert(NULL!=view_name);
	str = g_strdup_printf ( GCONF_PATH_TOTAL, view_name );
	/* writes the # of columns*/
	if(!gconf_client_set_int (gconf_client, str, g_list_length(columns_list), NULL ))
		ok = FALSE;
	g_free(str);

	for (l = columns_list; NULL != l; l = l->next) {
		ViewColumnDef *column = (ViewColumnDef*) l->data ;
		gchar *str_key = g_strdup_printf ( GCONF_PATH_COLUMN_N, view_name, column_index );
		if(!view_columns_save_conf( column, gconf_client, str_key ))
			ok = FALSE ;
		g_free(str_key);
		column_index ++;
	}
	return ok ;
}

gboolean
task_columns_load_configuration( GConfClient *gconf_client, const gchar *view_name, GList **columns_list, int *num_columns )
{
	GList    *columns = NULL;
	gboolean  ok = TRUE;
	int	  column_index = 0;
	gchar    *str=NULL;

	g_assert(NULL!=view_name);
	str = g_strdup_printf ( GCONF_PATH_TOTAL, view_name );
	*num_columns = gconf_client_get_int ( gconf_client, str, NULL  );
	g_free(str);

	for ( column_index = 0 ; column_index < *num_columns ; column_index++ ) {
		ViewColumnDef *column = view_column_new_void();
		if(NULL==column)
			ok = FALSE ;
		else
		{
			gchar *base_path = g_strdup_printf ( GCONF_PATH_COLUMN_N, view_name, column_index );
			if( !view_columns_read_conf( column, gconf_client, base_path ) )
				ok = FALSE ;
			g_free(base_path);
			columns = g_list_append( columns, column );
		}
	}
	if(!ok)
	{
		*num_columns = 0; 
		*columns_list = NULL;
		if(NULL!=columns)
			view_columns_end( columns );
	} else
	{
		*columns_list = columns ;
	}
	return ok ;
}
