/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2005  Luca Bellonda <lbell at tsc4.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __PLANNER_COL_SPC_H__
#define __PLANNER_COL_SPC_H__
#define	TV_ALIGN_LEFT	(0)
#define	TV_ALIGN_CENTER	(0.5)
#define	TV_ALIGN_RIGHT	(1)

#include <gconf/gconf-client.h>

/* Task Columns */
enum _ColTaskEnum {
		COL_TM_INVALID=-1,	/* Invalid column */
		COL_TM_START_BASE_COLUMN=0,/* the base of the enumeration*/
	COL_TM_WBS=COL_TM_START_BASE_COLUMN,
	COL_TM_NAME,
	COL_TM_START,
	COL_TM_FINISH,
	COL_TM_DURATION,
	COL_TM_WORK,
	COL_TM_SLACK,
	COL_TM_WEIGHT,
	COL_TM_EDITABLE,
	COL_TM_TASK,
	COL_TM_COST,
	COL_TM_RESOURCE,
	COL_TM_CONSTRAINT,
	COL_TM_NOTE,
	COL_TM_PERCENTDONE,	//TODO
	COL_TM_CARDINE,		//TODO
	COL_TM_FIXEDSTARTDATE,	//TODO
	COL_TM_STATEICON,	//TODO FORSE TOGLIERE
		COL_TM_NUM_COLS
} ;
typedef	enum _ColTaskEnum ColTaskEnum ;


struct _ViewColumnDef {
	int       uid;       /* internal identifier of the column */
	gchar    *title;    /* the column title */
	gboolean  editable;  /* editable flag */
	gfloat    alignment; /* the alignment */
	int       width;     /* width of the column */
};

typedef	struct _ViewColumnDef ViewColumnDef;

ViewColumnDef *view_column_new_void(void);
ViewColumnDef *view_column_new( const ColTaskEnum uid, const gchar *title, const gboolean editable, const gfloat alignment, const int width );
void view_column_set_title( ViewColumnDef *self, const gchar *newTitle );
gchar* view_column_get_title_transl_dup( ViewColumnDef *self );
ViewColumnDef *view_column_clone(ViewColumnDef *self );
void view_column_end( ViewColumnDef *self );
void view_column_def_init( ViewColumnDef *self, const ColTaskEnum uid, const gchar *title, const gboolean editable, const gfloat alignment, const int width );
const gchar *view_column_get_type_string_const(ViewColumnDef *self); /* returns the textual description of the column */

GList *view_columns_duplicate(GList *col_list);
void view_columns_end(GList *col_list);

GList *view_columns_get_default_gantt_init(void);
GList *view_columns_get_default_task_init(void);
/* returns a new list with ALL the columns (for the edit dialog) */
GList *view_column_get_all(void);

const gchar *view_column_get_default_label(const ColTaskEnum uid);

gboolean view_columns_save_conf( ViewColumnDef *self, GConfClient *gconf_client, const gchar * base_path );
/* It deletes the column if the read value is invalid */
gboolean view_columns_read_conf( ViewColumnDef *self, GConfClient *gconf_client, const gchar * base_path );

gboolean task_columns_load_configuration( GConfClient *gconf_client, const gchar *view_name, GList **columns_list, int *num_columns );
gboolean task_columns_save_configuration( GConfClient *gconf_client, const gchar *view_name, GList *columns_list );


#endif /* __PLANNER_COL_SPC_H__ */

