/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2005 Luca Bellonda lbell-at-tsc4.com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <config.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <libgnome/gnome-i18n.h>
#include <glade/glade.h>
#include <gtk/gtk.h>
#include <libplanner/mrp-object.h>
#include <libplanner/mrp-project.h>
#include "planner-cell-renderer-list.h"
#include "planner-assignment-model.h"
#include "planner-predecessor-model.h"
#include "planner-task-cmd.h"
#include "planner-task-columns-dialog.h"

/* debug*/
#define	D(x)	

enum {
        EDIT_TASK_COLUMNS_ID,
        EDIT_TASK_COLUMNS_TITLE,
	/*EDIT_TASK_COLUMNS_PTR,*/
        NUMBER_OF_EDIT_TASK_COLUMNS_COLS
};

typedef struct {
	PlannerWindow *main_window;
	GtkWidget     *dialog;
	GtkWidget     *w_shown_columns_list;
	GtkWidget     *w_hidden_columns_list;
	GtkWidget     *w_move_up_button;
	GtkWidget     *w_move_down_button;
	GtkWidget     *w_add_button;
	GtkWidget     *w_remove_button;
	GList         *list_shown_columns;
	GList         *list_columns;
} ECDialogData ;


static void      task_columns_dialog_buttons_enable( ECDialogData *dialog, const gboolean enabled );
static void      task_columns_check_buttons_state( ECDialogData *dialog );

/* Keep the dialogs here so that we can just raise the dialog if it's
 * opened twice for the same task.
 */
#define DIALOG_GET_DATA(d) g_object_get_data ((GObject*)d, "data")

/* Builds the availables columns dialog
 */
static void
task_columns_dialog_load_popup(ECDialogData *self)
{
	GtkWidget     *menu;
	GtkWidget     *menu_item;
	GList         *l,*l_inner;
	gboolean       is_column_found;

       	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (self->w_hidden_columns_list));
	if (menu) {
		gtk_widget_destroy (menu);
		menu = NULL;
	}

	menu = gtk_menu_new ();
	/* foreach column, check if it is visible (a Hash table will be better, but the # of columns is low ) */
	for (l = self->list_columns; NULL != l; l = l->next) {
		ViewColumnDef* col_def = (ViewColumnDef*) l->data ;
		is_column_found = FALSE ;
		for (l_inner = self->list_shown_columns; NULL != l_inner; l_inner = l_inner->next) {
			ViewColumnDef* col_def_visible = (ViewColumnDef*) l_inner->data ;
			if( col_def_visible->uid == col_def->uid ) {
				is_column_found = TRUE;
				break;
			}
		}
		if( !is_column_found ) {
			menu_item = gtk_menu_item_new_with_label (view_column_get_type_string_const(col_def));
			gtk_widget_show (menu_item);
			gtk_menu_append (GTK_MENU (menu), menu_item);
			g_object_set_data (G_OBJECT (menu_item), "data", col_def );
		}
	}
	gtk_widget_show (menu);
	gtk_option_menu_set_menu (GTK_OPTION_MENU (self->w_hidden_columns_list), menu);
}
/*
static void debug_show_list(gchar *sit, GList *list )
{
	GList *l_inner ;
	int i =0;
	printf( "%s\n", sit);
	
	for (l_inner = list; NULL != l_inner; l_inner = l_inner->next) {
		printf( " %d- pos ptr: %lx data:%lx\n", i, l_inner, l_inner->data );
		i++;
	}
	printf( "end\n");
	printf( "\n");
}
*/
static void
task_columns_dialog_remove_col_cb(GtkWidget *button, ECDialogData *dialog)
{
	GtkTreeSelection    *selection;
	GtkTreeIter          iter;
	ViewColumnDef       *pDef = NULL ;
	GtkTreeModel        *model;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->w_shown_columns_list));

	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
		return;
	}

	model = gtk_tree_view_get_model(GTK_TREE_VIEW (dialog->w_shown_columns_list));
	if(NULL==model)
		return ;
	gtk_tree_model_get (model, &iter,
			    0, &pDef,
			    -1);
	/* removes the selected entry from the list */
	if(NULL!=pDef)
	{
D(mostra_lista("prima", dialog->list_shown_columns );)
		dialog->list_shown_columns = g_list_remove( dialog->list_shown_columns, pDef );
D(mostra_lista("dopo", dialog->list_shown_columns );)
		gtk_list_store_remove( GTK_LIST_STORE(model), &iter );
		view_column_end( pDef );
		task_columns_check_buttons_state( dialog );
	}
D(mostra_lista("load", dialog->list_shown_columns );)
	task_columns_dialog_load_popup(dialog);
}

static void
task_columns_dialog_resort_cols(ECDialogData *dialog)
{
	GtkTreeIter          iter;
	ViewColumnDef       *pDef = NULL ;
	GtkTreeModel        *model;
	gboolean             flag_iter;
	GList               *col_list = NULL;

	model = gtk_tree_view_get_model(GTK_TREE_VIEW (dialog->w_shown_columns_list));
	if(NULL==model)
		return ;
	flag_iter = gtk_tree_model_get_iter_first (model, &iter);
	while (flag_iter)
	{
		gtk_tree_model_get (model, &iter,
				    0, &pDef,
				    -1);
	        col_list = g_list_append (col_list, pDef );
		flag_iter = gtk_tree_model_iter_next (model, &iter);
	}
	g_list_free(dialog->list_shown_columns);
	dialog->list_shown_columns = col_list ;

}

static void
task_columns_dialog_move_down_cb(GtkWidget *button, ECDialogData *dialog)
{
	GtkTreeSelection    *selection;
	GtkTreeIter          iter,iter_next;
	GtkTreeModel        *model;
	gchar               *path ;
	GtkTreePath         *path_ptr;
	int                  nPos ;
	char                 szPos[40];

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->w_shown_columns_list));

	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
		return;
	}
	model = gtk_tree_view_get_model(GTK_TREE_VIEW (dialog->w_shown_columns_list));
	if(NULL==model)
		return ;
	/* it is a list store so we know about the string */
	path = gtk_tree_model_get_string_from_iter ( model, &iter );
	nPos = atoi(path);
	g_free(path);
	nPos++;
	sprintf( szPos, "%d", nPos );
	path_ptr = gtk_tree_path_new_from_string (szPos);
	if(NULL!=path_ptr)
	{
		gboolean is_set;
		is_set = gtk_tree_model_get_iter ( model, &iter_next, path_ptr );
		gtk_tree_path_free (path_ptr);
		if(is_set)
			gtk_list_store_swap( GTK_LIST_STORE(model), &iter, &iter_next ) ;
	}
	task_columns_dialog_resort_cols(dialog);
	task_columns_check_buttons_state( dialog );
}

static void
task_columns_dialog_move_up_cb(GtkWidget *button, ECDialogData *dialog)
{
	GtkTreeSelection    *selection;
	GtkTreeIter          iter,iter_next;
	GtkTreeModel        *model;
	gchar               *path ;
	GtkTreePath         *path_ptr;
	int                  nPos ;
	char                 szPos[40];

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->w_shown_columns_list));

	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) {
		return;
	}
	model = gtk_tree_view_get_model(GTK_TREE_VIEW (dialog->w_shown_columns_list));
	if(NULL==model)
		return ;
	/* it is a list store so we know about the string */
	
	path = gtk_tree_model_get_string_from_iter ( model, &iter );

	nPos = atoi(path);
	g_free(path);
	nPos--;
	if(nPos >=0)
	{
		sprintf( szPos, "%d", nPos );
		path_ptr = gtk_tree_path_new_from_string (szPos);
		if(NULL!=path_ptr)
		{
			gboolean is_set;
			is_set = gtk_tree_model_get_iter ( model, &iter_next, path_ptr );
			gtk_tree_path_free (path_ptr);
			if(is_set)
				gtk_list_store_swap( GTK_LIST_STORE(model), &iter, &iter_next ) ;
		}
	}
	task_columns_dialog_resort_cols(dialog);
	task_columns_check_buttons_state( dialog );
}

static void
task_columns_dialog_add_col_cb(GtkWidget *button, ECDialogData *dialog)
{
	GtkTreeIter    iter;
	ViewColumnDef *pDef = NULL, *cloned_col ;
	GtkTreeModel  *model;
	GtkWidget     *menu;
	GtkWidget     *menu_item;
	
       	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (dialog->w_hidden_columns_list));
	if (NULL==menu)
		return ;
	menu_item = gtk_menu_get_active (GTK_MENU (menu));
	if(NULL==menu_item)
		return ;

	model = gtk_tree_view_get_model(GTK_TREE_VIEW (dialog->w_shown_columns_list));
	if(NULL==model)
		return ;
	pDef = g_object_get_data (G_OBJECT (menu_item), "data" );
	if(NULL==pDef)
		return ;
	cloned_col = view_column_clone(pDef) ;
	if(NULL==cloned_col)
		return ;
	dialog->list_shown_columns = g_list_append( dialog->list_shown_columns, cloned_col );
	gtk_list_store_append (GTK_LIST_STORE(model), &iter);
	gtk_list_store_set ( GTK_LIST_STORE(model),
			     &iter,
			     0/*EDIT_TASK_PTR*/, cloned_col,
			     -1);
	task_columns_dialog_load_popup(dialog);
	task_columns_check_buttons_state( dialog );
}

/* the list of visible columns */
static void
task_columns_dialog_label_edited(GtkCellRendererText *cell,
		       gchar               *path_string,
		       gchar               *new_text,
		       gpointer             data)
{
	ECDialogData        *dialog;
	GtkTreeView         *tree;
	GtkTreeModel        *model;
	GtkTreePath         *path;
	GtkTreeIter          iter;
	ViewColumnDef       *pDef = NULL ;

	dialog = (ECDialogData*)data;
	tree = GTK_TREE_VIEW(dialog->w_shown_columns_list);
	model = gtk_tree_view_get_model (tree);

	path = gtk_tree_path_new_from_string (path_string);
	gtk_tree_model_get_iter (model, &iter, path);
D(printf("path is %s=%s\n", path_string, new_text );)
	gtk_tree_model_get (model, &iter, 0, &pDef, -1);
D(printf("ptr is %lx\n", pDef );)
	g_assert(NULL!=pDef);
	view_column_set_title( pDef, new_text );
	gtk_tree_path_free (path);
}

static void
task_columns_dialog_get_data_showncol(GtkTreeViewColumn *tree_column,
				      GtkCellRenderer   *cell,
				      GtkTreeModel      *tree_model,
				      GtkTreeIter       *iter,
				      gpointer           data)
{
	ViewColumnDef	*pDef =NULL ;
	gchar           *text=NULL;
	gtk_tree_model_get (tree_model, iter, 0, &pDef, -1);
	g_assert(NULL!=pDef);
	switch((int)data)
	{
	case EDIT_TASK_COLUMNS_ID:
		if(NULL!=pDef)
			text=g_strdup(view_column_get_type_string_const(pDef));
		else
			text=g_strdup("null");
		break;
	case EDIT_TASK_COLUMNS_TITLE:
		if(NULL!=pDef)
		{
			text=view_column_get_title_transl_dup(pDef);
		}else
			text=g_strdup("null");
		break;
	default:
		g_warning ("Type not implemented.");
		text=g_strdup("?");
		break;
	}
	g_object_set (cell, "text", text, NULL);
	g_free(text);
}

static void
task_columns_dialog_buttons_enable( ECDialogData *dialog, const gboolean enabled )
{
	gtk_widget_set_sensitive(dialog->w_move_up_button, enabled );
	gtk_widget_set_sensitive(dialog->w_move_down_button, enabled );
	gtk_widget_set_sensitive(dialog->w_remove_button, enabled );
}
static void
task_columns_check_buttons_state( ECDialogData *dialog )
{
	GtkTreeSelection  *selection;
	GtkTreeIter        iter;
	GtkTreeModel      *model;
	gboolean           enabled = FALSE ;

	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (dialog->w_shown_columns_list));
	if(gtk_tree_selection_get_selected( selection, &model, &iter ) )
		enabled = TRUE;
	task_columns_dialog_buttons_enable( dialog, enabled );
}
static void
task_columns_dialog_changed_sel_cb( GtkTreeSelection *selection, ECDialogData *dialog ) 
{
	task_columns_check_buttons_state( dialog );
}

static void
task_columns_dialog_setup_columns_list (ECDialogData *data)
{
	GtkTreeView       *tree;
	GtkTreeViewColumn *col;
	GtkCellRenderer   *cell;
	GtkTreeModel      *model;
	GtkListStore      *store;
	GtkTreeIter        iter;
	GtkTreeSelection  *selection;

	tree = GTK_TREE_VIEW (data->w_shown_columns_list);
	selection = gtk_tree_view_get_selection (tree);
	gtk_tree_selection_set_mode (selection,
				     GTK_SELECTION_SINGLE);
	g_signal_connect ( selection,
			  "changed",
			  G_CALLBACK (task_columns_dialog_changed_sel_cb),
			  data );
	GList             *l;
	store = gtk_list_store_new (1/*NUMBER_OF_EDIT_TASK_COLUMNS_COLS*/, G_TYPE_POINTER);
	model = GTK_TREE_MODEL (store);
	gtk_tree_view_set_model (tree, model);

	for (l = data->list_shown_columns; NULL != l; l = l->next) {
		ViewColumnDef* col_def = (ViewColumnDef*) l->data ;
		gtk_list_store_append (store, &iter);

		gtk_list_store_set (store,
				    &iter,
				    0/*EDIT_TASK_PTR*/, col_def,
				    -1);
	}
	g_object_unref( store );
	/* Type */
	cell = gtk_cell_renderer_text_new ();
	g_object_set (cell, "editable", FALSE, NULL);
	col = gtk_tree_view_column_new_with_attributes (_("Type"), cell, NULL);
	gtk_tree_view_column_set_cell_data_func (col, cell, 
						 (GtkTreeCellDataFunc) task_columns_dialog_get_data_showncol,
						 (gpointer)EDIT_TASK_COLUMNS_ID, 0);
	gtk_tree_view_append_column (tree, col);

	cell = gtk_cell_renderer_text_new ();
	g_object_set (cell, "editable", TRUE, NULL);
	/* Label */
	col = gtk_tree_view_column_new_with_attributes (_("Label"),cell, NULL);
	gtk_tree_view_column_set_cell_data_func (col, cell, 
						 (GtkTreeCellDataFunc) task_columns_dialog_get_data_showncol,
						 (gpointer)EDIT_TASK_COLUMNS_TITLE, 0);
	g_signal_connect (cell,
			  "edited",
			  G_CALLBACK (task_columns_dialog_label_edited),
			  data);
	gtk_tree_view_append_column (tree, col);
}

static void
columns_dialog_update_title (ECDialogData *data, const PlannerViewColumnsEnum view_type )
{
	gchar *title;

	if ( COLUMNS_TASK_VIEW == view_type ) {
		title = g_strdup (_("Edit task view columns"));
	} else {
		title = g_strdup (_("Edit gantt view columns"));
	}

	gtk_window_set_title (GTK_WINDOW (data->dialog), title);

	g_free (title);
}	


static void
task_dialog_destroy_cb (GtkWidget  *parent,
			ECDialogData *data)
{
}

GList *
planner_edit_task_columns (          PlannerWindow           *window,
			       const PlannerViewColumnsEnum   view_type,
			             GList                   *actual_shown_columns )

{
	ECDialogData *data;
	GladeXML     *glade;
	GtkWidget    *dialog;
	GList        *resList = NULL;
	gboolean      isReturnList = FALSE;
	
	glade = glade_xml_new (GLADEDIR "/task-columns-dialog.glade",
			       NULL,
			       GETTEXT_PACKAGE);
	if (!glade) {
		g_warning ("Could not create task columns dialog.");
		return NULL;
	}

	dialog = glade_xml_get_widget (glade, "task_columns_dialog");

	data = g_new0 (ECDialogData, 1);
	/* changes are done on the duoplicate list */
	data->list_shown_columns = view_columns_duplicate(actual_shown_columns) ;
	data->list_columns= view_column_get_all();

	data->main_window = window;
	data->dialog = dialog;

	g_signal_connect (dialog,
			  "destroy",
			  G_CALLBACK (task_dialog_destroy_cb),
			  data);
	
	data->w_shown_columns_list   = glade_xml_get_widget (glade, "columns_tree");
	data->w_hidden_columns_list  = glade_xml_get_widget (glade, "columnstoshow");
	data->w_move_up_button       = glade_xml_get_widget (glade, "moveup");
	data->w_move_down_button     = glade_xml_get_widget (glade, "movedown");
	data->w_add_button           = glade_xml_get_widget (glade, "add_columns");
	data->w_remove_button        = glade_xml_get_widget (glade, "remove");

	g_assert(NULL!=data->w_shown_columns_list);
	g_assert(NULL!=data->w_hidden_columns_list );
	g_assert(NULL!=data->w_move_up_button);
	g_assert(NULL!=data->w_move_down_button);
	g_assert(NULL!=data->w_remove_button);
	
	gtk_widget_set_size_request( data->w_shown_columns_list, -1, 100 );
	task_columns_dialog_setup_columns_list (data);
	task_columns_dialog_load_popup(data);
	g_signal_connect ( data->w_move_up_button,
			  "clicked",
			  G_CALLBACK (task_columns_dialog_move_up_cb),
			  data);
	g_signal_connect ( data->w_move_down_button,
			  "clicked",
			  G_CALLBACK (task_columns_dialog_move_down_cb),
			  data);
	g_signal_connect ( data->w_remove_button,
			  "clicked",
			  G_CALLBACK (task_columns_dialog_remove_col_cb),
			  data );
	g_signal_connect ( data->w_add_button,
			  "clicked",
			  G_CALLBACK (task_columns_dialog_add_col_cb),
			  data );

	g_object_set_data_full (G_OBJECT (dialog),
				"data", data,
				NULL/*g_free*/);

	task_columns_check_buttons_state( data );

	columns_dialog_update_title(data, view_type );
	resList = NULL ;
	if(NULL!=dialog)
	{
		gint result = gtk_dialog_run (GTK_DIALOG (dialog));
		if(GTK_RESPONSE_OK==result)
    		{
			/* build the resulting list */
			resList = data->list_shown_columns;
			isReturnList = TRUE;
		}
		gtk_widget_destroy (dialog);
	}
	if( !isReturnList && (NULL!=data->list_shown_columns) )
	{
		view_columns_end(data->list_shown_columns);
	}
	view_columns_end(data->list_columns);
	g_free(data);
	return resList;
}
